;;; daemon-tools
;;;
;;; Copyright (c) 2007  Hans Bulfone <jsb@nil.at>
;;; All rights reserved.
;;;
;;; Redistribution and use in source and binary forms, with or without
;;; modification, are permitted provided that the following conditions are met:
;;;
;;;     * Redistributions of source code must retain the above copyright notice,
;;;       this list of conditions and the following disclaimer.
;;;     * Redistributions in binary form must reproduce the above copyright
;;;       notice, this list of conditions and the following disclaimer in the
;;;       documentation and/or other materials provided with the distribution.
;;;     * Neither the name of the author nor the names of his contributors may
;;;       be used to endorse or promote products derived from this software
;;;       without specific prior written permission.
;;;
;;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
;;; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
;;; THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
;;; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
;;; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
;;; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
;;; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
;;; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
;;; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
;;; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
;;; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

(define-extension daemon-tools)
(declare
 (export daemon:openlog daemon:syslog daemon:ize)
 (foreign-declare "#include <syslog.h>"))
(require-extension posix)

(define-macro (def-c-constants prefix . constants)
  `(begin
     ,@(append-map
        (lambda (x)
          (let ((scm-name (string->symbol
                           (string-append
                            prefix
                            (string-translate x "_" "-")))))
            `((define ,scm-name (foreign-value ,x integer))
              (declare (export ,scm-name)))))
        constants)))

(def-c-constants "daemon:"
  "LOG_CONS" "LOG_NDELAY" "LOG_NOWAIT" "LOG_ODELAY" "LOG_PID"
  "LOG_AUTHPRIV" "LOG_CRON" "LOG_DAEMON" "LOG_KERN"
  "LOG_LOCAL0" "LOG_LOCAL1" "LOG_LOCAL2" "LOG_LOCAL3" "LOG_LOCAL4"
  "LOG_LOCAL5" "LOG_LOCAL6" "LOG_LOCAL7"
  "LOG_LPR" "LOG_MAIL" "LOG_NEWS" "LOG_SYSLOG" "LOG_USER" "LOG_UUCP"
  "LOG_EMERG" "LOG_ALERT" "LOG_CRIT" "LOG_ERR" "LOG_WARNING"
  "LOG_NOTICE" "LOG_INFO" "LOG_DEBUG")

;; WARNING: openlog leaks memory if ident is not #f
(define daemon:openlog
  (foreign-lambda*
   void ((c-string ident) (integer option) (integer facility))
   "openlog(((ident)?strdup(ident):NULL), option, facility);"))

(define %syslog
  (foreign-lambda*
   void ((integer priority) (nonnull-c-string message))
   "syslog(priority, \"%s\\n\", message);"))

(define (daemon:syslog priority message . args)
  (%syslog priority (apply sprintf message args)))

(define (daemon:ize)
  (change-directory "/")
  (let ((fd-r (file-open "/dev/null" open/rdonly))
        (fd-w (file-open "/dev/null" open/wronly)))
    (duplicate-fileno fd-r 0)
    (duplicate-fileno fd-w 1)
    (file-close fd-r)
    (file-close fd-w))
  (let ((child-pid (process-fork)))
    (if (not (zero? child-pid))
        (exit 0)))
  (create-session)
  (duplicate-fileno 1 2)
  (void))
